package com.energy.manage.service.service.analysis;

import com.alibaba.fastjson.JSONArray;
import com.energy.manage.common.reponse.ResultResp;
import com.energy.manage.service.domain.dto.analysis.AnalysisDto;
import com.energy.manage.service.domain.dto.analysis.AnalysisResultDto;
import com.energy.manage.service.domain.vo.analysis.AnalysisDetailVo;
import com.energy.manage.service.domain.vo.analysis.AnalysisResultShowVo;
import com.energy.manage.service.domain.vo.analysis.AnalysisResultVo;
import com.energy.manage.service.domain.vo.analysis.AnalysisTypeShortVo;
import com.energy.manage.service.domain.vo.windenginegroup.WindEngineGroupShortVo;
import org.apache.ibatis.mapping.ResultMap;
import org.springframework.web.bind.annotation.RequestParam;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Set;

/**
 * 分析服务
 * @author chy
 * @date 2024/5/23 12:59
 * @desc
 */
public interface AnalysisService {

    /**
     * 分析结果列表
     * @param analysisResultDto
     * @return
     */
    List<AnalysisResultVo> analysisResultList(AnalysisResultDto analysisResultDto);


    /**
     * 分析
     * @param analysisDto
     * @return
     */
    ResultResp analysis(AnalysisDto analysisDto);


    /**
     * 分析编辑查询
     * @param batchCode
     * @return
     */
    AnalysisResultShowVo analysisEditQuery(String batchCode);

    /**
     * 查询已经分析的分析类型和风机信息
     * @param batchCode
     * @return
     */
    Set<AnalysisTypeShortVo> queryAnalysisedType(String batchCode);

    /**
     * 查询已经分析的风机信息
     * @param batchCode
     * @param analysisTypeCode
     * @return
     */
    Set<WindEngineGroupShortVo> queryAnalysisedEngine(String batchCode, String analysisTypeCode);

    /**
     * 分析详情
     * @param batchCode
     * @param analysisTypeCode
     * @param fieldEngineCode
     * @param isIntranet
     * @return
     */
    List<AnalysisDetailVo> analysisDetail(String batchCode, String analysisTypeCode, String fieldEngineCode, boolean isIntranet);


    /**
     * 分析中检查
     * @return
     */
    boolean checkAnalysising();

    /**
     * 开启/关闭自动分析
     * @param batchCode
     * @param onOffCall
     * @return
     */
    int onOffAutoAnalysis(String batchCode, Integer onOffCall);


    /**
     * 分析类型结果图配置接口
     * @return
     */
    JSONArray queryAnalysisTypeConfig();


    /**
     * 全部文件下载查询接口
     * @param fieldCode
     * @param batchCode
     * @return
     */
    List<String> queryDownloadFile(String fieldCode, String batchCode);


    /**
     * 文件现在
     * @param batchCode
     * @param fieldCode
     * @param analysisTypeCode
     * @return
     */
    ResultResp downloadFile(String fieldCode, String batchCode, String analysisTypeCode, HttpServletResponse response);



}
